import requests
import csv
import re


# **********************************  My DEMO APP API INFORMATION ************************************
location = "ENTER YOUR LOCATION INFO HERE"
authtoken = "ENTER YOUR AUTH TOKEN HERE"

# **********************************  VARIABLES for API CALLS ****************************************
meridian_url = "https://edit.meridianapps.com/api/locations/"
locations_api = "https://edit.meridianapps.com/api/locations/" + str(location)
beacons_api = locations_api + "/beacons?page_size="   # without page size
pagesize = 200 # default page size to be set to 200 if long list
maps_api_url = locations_api + "/maps"
beacon_url_prefix = "https://edit.meridianapps.com/w/location/" + location + "/beacon/"

payload = ""
headers = {
    'Content-Type': "application/x-www-form-urlencoded",
    'Authorization': "Token" + str(authtoken),
    'cache-control': "no-cache"
    }

# ********************************** GLOBAL VARIABLES for PROGRAM LOGIC *************************
global_beacons_list = [] # global list of beacon dictionary. This will be filled by get_beacons function
global_maps_list = [] # global list of maps dictionary. This will be filled by get_maps function


# ********************************** get_beacons Function ***************************************
# Recursive Function that loops over all the beacons and fills the result in the global variable
# global_beacons_list
def get_beacons(beacons_api_url,api_call_counter,pagesize):
    try:
        response = requests.request("GET", beacons_api_url, data=payload, headers=headers)
        jparse = response.json()
        beacons_data = jparse['results']
        global_beacons_list.extend(beacons_data)
        print("API Call "+str(api_call_counter)+ " - Discovered Beacon(s): " + str(len(global_beacons_list)))
        api_call_counter += 1
        if str(jparse['next']) != 'None':
            beacons_api_url = jparse['next']
            get_beacons(beacons_api_url, api_call_counter,pagesize)
        else:
            return
    except Exception as e:
        print("An error happened while calling get_beacons. Error details are " + str(e))
        return

# ********************************** get_maps function ******************************************
# Recursive Function to Get All Maps at a particular location
# Function saves the result in the global_maps_list
def get_maps(maps_api_url):
    try:
        maps_request = requests.get(maps_api_url, headers=headers)
        maps_parsed = maps_request.json()
        global_maps_list.extend(maps_parsed['results'])
        if str(maps_parsed['next']) != 'None':
            maps_api_url = maps_parsed['next']
            get_maps(maps_api_url)
        else:
            return
    except Exception as e:
        print("An error happened while calling get_maps. Error details are " + str(e))
        return


# ********************************** beacon_power Function **************************************
def print_beacon_power(beacons_data, limit):
    ap_regex = re.compile('AP')  # regex to find an AP beacon type
    ap_beacon_count = 0  # count the APs which aren't battery powered
    battery_beacon_count = 0  # count the total number of battery beacons
     # beacons_count_full_battery = 0  # count the beacons with a full battery
    beacons_count_low_battery = 0  # count the beacons with a low battery (as defined in low)
    beacons_count_between_low_and_full = 0  # count the beacons with less than 100% battery (not inclusive of low_count)
    beaconinfo = {} # Dictionary for beacons
    low_beacons = [] # list of beacons with low batteries (below or equal to limit)

    for beacon in beacons_data:
        for map in global_maps_list:
            if map['id'] == beacon['map']:
                beaconinfo = {'id': (beacon['id']),'battery_level': (beacon['battery_level']),'hw_type': (beacon['hw_type']),'map': map['name']}
        if re.search(ap_regex, beacon['hw_type']):
            ap_beacon_count += 1
        else:
            battery_beacon_count += 1
            if beacon['battery_level'] <= limit:
                low_beacons.append(beaconinfo)
                beacons_count_low_battery += 1
            else:
                beacons_count_between_low_and_full+= 1


    # Printing Summary of Results
    print("\n\nTotal Beacons {} - AP Beacons {} - Battery Beacons {}".format(len(beacons_data),ap_beacon_count,battery_beacon_count))



    # Printing Beacons with low Battery and saving same result to file named beacons-list.csv
    print("{:-^141s}".format("Beacons with Battery Level below or equal to "+str(limit)))
    print("{0:<13}\t{1:<13}\t{2:<13}\t{3:<13}\t{4}".format("Beacon-ID","Power-Level","TYPE","Floor","URL"))
    with open('beacons-list.csv', 'w', newline='') as writefile:
       writer =csv.writer(writefile,delimiter=",")
       writer.writerow(['Beacon-ID','Power-Level','TYPE','Floor','URL'])
       for beacon in low_beacons:
          print("{0:<13}\t{1:<13}\t{2:<13}\t{3:<13}\t{4}".format(beacon['id'],beacon['battery_level'],beacon['hw_type'],beacon['map'],beacon_url_prefix + beacon['id']))
          writer.writerow([beacon['id'],beacon['battery_level'],beacon['hw_type'],beacon['map'],beacon_url_prefix + beacon['id']])
       writefile.close()


# **********************************  MAIN FUNCTION **************************************************
if __name__ =="__main__":
    try:
        # ********************************  Get Location Information *********************************
        location_info = requests.get(locations_api, headers=headers)
        loc_parsed = location_info.json()
        print("\nLocation Name:", loc_parsed['name'])

        # **********************************  Get Maps Information ***********************************
        print("Getting info about the maps at this location...  This may take some time... \n")
        get_maps(maps_api_url)

        # ******************************** Get Beacons Information ***********************************
        beacons_api_url = beacons_api + str(pagesize)
        print("Collecting Beacons Information use this Beacon API URL:", beacons_api_url)
        api_call_counter = 1
        get_beacons(beacons_api_url,api_call_counter,pagesize)

        # ********** Print and Save Beacons whose battery level is below the provided limit **********
        limit = 30 #All battery based beacons below or equal to limit will be printed
        print_beacon_power(global_beacons_list,limit)


    except Exception as e:
        print("An error happened while running the main function. Error details are " + str(e))
    exit()